package xyz.veronie.bgg.result;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;

import javax.annotation.PostConstruct;
import javax.inject.Singleton;

import org.eclipse.e4.core.di.annotations.Creatable;

import xyz.veronie.bgg.localdb.LocalDbAdapterService;

@Creatable
@Singleton
public class ThingProvider {
	private LocalDbAdapterService localDbAdapterService;
	
	/// list of things. Each ID is expected to exist exactly once.
	private List<ThingMetaData> thingMetas;

	public ThingProvider() {}
	
	@PostConstruct
	public void init() {
		localDbAdapterService = new LocalDbAdapterService();
		thingMetas = new ArrayList<ThingMetaData>();
	}
	
	/// replace current list with new list
	public void replaceThingMetas(ArrayList<ThingMetaData> metas) {
		this.thingMetas = metas;
	}
	
	/// add things from argument to current list iff their ID does not exist yet
	public void addThingMetas(ArrayList<ThingMetaData> metas) {
		for(ThingMetaData tmd : metas) {
			Boolean exists = false;
			for(ThingMetaData thisTmd : this.thingMetas) {
				if(tmd.getId() == thisTmd.getId()) {
					exists = true;
				}
			}
			if(!exists) {
				this.thingMetas.add(tmd);
			}
		}
	}
	
	
	// helper function for subtractThingMetas
	private static Predicate<ThingMetaData> thingEqual(final ThingMetaData other) 
	{
	    return p -> p.getId() == other.getId();
	}
	
	/// remove the things in the argument from the current list of things (using their ID)
	public void subtractThingMetas(ArrayList<ThingMetaData> metas) {
		for(ThingMetaData tmd : metas) {
			this.thingMetas.removeIf(thingEqual(tmd));
		}
	}

	/// keep only things that exist in both argument list and current list (by ID)
	public void intersectThingMetas(ArrayList<ThingMetaData> metas) {
		List<ThingMetaData> cpMetas = new ArrayList<ThingMetaData>(this.thingMetas);
		this.thingMetas.clear();
		for(ThingMetaData tmd : metas) {
			for(ThingMetaData thisTmd : cpMetas) {
				if(tmd.getId() == thisTmd.getId()) {
					this.thingMetas.add(tmd);
				}
			}
		}
	}
	
	/// store current list in DB
	public void storeList() {
		localDbAdapterService.storeThingList(this.thingMetas, "TestList");
	}
	
	
	// return the current list of ids
	public List<ThingMetaData> getThingMetas() {
        return thingMetas;
    }
	
//	@Inject
//	@Optional
//	private void subscribeTopicTagResult
//	    (@UIEventTopic(EventConstants.TOPIC_TAG_RESULT)
//	        String empty) {
//		System.out.println("TOPIC_TAG_RESULT: Tag result now.");
//		this.tagResult();
//	}
	
}
